#include <qlayout.h>
#include <qpushbutton.h>
#include <qbuttongroup.h>
#include <dspaction.h>
#include <dialset.h>
#include <dialpad.h>

DialPad::DialPad( DialSet *initset, QWidget *parent, const char *name )
	: QWidget( parent, name )
{
	curset = initset;

	QGridLayout *grid = new QGridLayout( this, 3, 5 );

	grid->setMargin( 1 );
	grid->setSpacing( 3 );
	grid->setResizeMode( QLayout::Fixed );

	buttons = new QButtonGroup( this );
	connect( buttons, SIGNAL( pressed( int ) ), this, SLOT( buttonPressed( int ) ) );
	connect( buttons, SIGNAL( released( int ) ), this, SLOT( buttonReleased( int ) ) );
	buttons->hide();

	// 1
	QPushButton *button = new QPushButton( this );
	buttons->insert( button, 0 );
	grid->addWidget( button, 0, 0 );

	// 2
	button = new QPushButton( this );
	buttons->insert( button, 1 );
	grid->addWidget( button, 0, 1 );

	// 3
	button = new QPushButton( this );
	buttons->insert( button, 2 );
	grid->addWidget( button, 0, 2 );

	// Pad A
	button = new QPushButton( this );
	buttons->insert( button, 12 );
	grid->addWidget( button, 0, 3 );

	// 4
	button = new QPushButton( this );
	buttons->insert( button, 3 );
	grid->addWidget( button, 1, 0 );

	// 5
	button = new QPushButton( this );
	buttons->insert( button, 4 );
	grid->addWidget( button, 1, 1 );

	// 6
	button = new QPushButton( this );
	buttons->insert( button, 5 );
	grid->addWidget( button, 1, 2 );

	// Pad B
	button = new QPushButton( this );
	buttons->insert( button, 13 );
	grid->addWidget( button, 1, 3 );

	// 7
	button = new QPushButton( this );
	buttons->insert( button, 6 );
	grid->addWidget( button, 2, 0 );

	// 8
	button = new QPushButton( this );
	buttons->insert( button, 7 );
	grid->addWidget( button, 2, 1 );

	// 9
	button = new QPushButton( this );
	buttons->insert( button, 8 );
	grid->addWidget( button, 2, 2 );

	// Pad C
	button = new QPushButton( this );
	buttons->insert( button, 14 );
	grid->addWidget( button, 2, 3 );

	// Pad *
	button = new QPushButton( this );
	buttons->insert( button, 9 );
	grid->addWidget( button, 3, 0 );

	// 0
	button = new QPushButton( this );
	buttons->insert( button, 10 );
	grid->addWidget( button, 3, 1 );

	// Pad #
	button = new QPushButton( this );
	buttons->insert( button, 11 );
	grid->addWidget( button, 3, 2 );

	// Pad D
	button = new QPushButton( this );
	buttons->insert( button, 15 );
	grid->addWidget( button, 3, 3 );

	// KP1
	button = new QPushButton( this );
	buttons->insert( button, 16 );
	grid->addWidget( button, 0, 4 );

	button = new QPushButton( this );
	buttons->insert( button, 20 );
	grid->addWidget( button, 0, 5 );

	button = new QPushButton( this );
	buttons->insert( button, 17 );
	grid->addWidget( button, 1, 4 );

	button = new QPushButton( this );
	buttons->insert( button, 21 );
	grid->addWidget( button, 1, 5 );

	button = new QPushButton( this );
	buttons->insert( button, 18 );
	grid->addWidget( button, 2, 4 );

	button = new QPushButton( this );
	buttons->insert( button, 22 );
	grid->addWidget( button, 2, 5 );

	button = new QPushButton( this );
	buttons->insert( button, 19 );
	grid->addWidget( button, 3, 4 );

	button = new QPushButton( this );
	buttons->insert( button, 23 );
	grid->addWidget( button, 3, 5 );

	renameKeys();
}

DialPad::~DialPad()
{
	if( curset != 0 ) {
		delete curset;
	}
}

void DialPad::setDialSet( DialSet *newset )
{
	if( curset != 0 ) {
		delete curset;
	}

	curset = newset;

	renameKeys();
	repaint();
}

void DialPad::hitKey( long id )
{
	if( curset == 0 ) return;

	DspAction *action = curset->playKey( id );

	if( action != 0 ) {
		playSound( action );
	}
}

void DialPad::buttonPressed( int id )
{
	if( curset == 0 ) return;

	curtone = curset->playKey( id + 1 );

	if( curtone != 0 ) {
		curtone->setLoop( true );
		playSound( curtone );
		keyPressed( id + 1 );
	}
}

void DialPad::buttonReleased( int )
{
	if( curtone != 0 ) {
		curtone->stop();
	}
}

void DialPad::renameKeys()
{
	DialKey *key;

	for( int i = 0; i < buttons->count(); i++ ) {

		if( curset != 0 ) {
			key = curset->getKey( i + 1 );
		} else {
			key = 0;
		}

		if( key != 0 ) {
			buttons->find( i )->setText( key->getName() );
		} else {
			buttons->find( i )->setText( " " );
		}
	}
}

void DialPad::showKeyCodes( bool showcodes )
{
	if( showcodes ) {
		for( int i = 0; i < buttons->count(); i++ ) {
			buttons->find( i )->setText( getKeyCode( i + 1 ) );
		}
	} else {
		renameKeys();
	}
}

QString DialPad::getKeyCode( long id )
{
	switch( id ) {
		case 1:		return "1";
		case 2:		return "2";
		case 3:		return "3";
		case 4:		return "4";
		case 5:		return "5";
		case 6:		return "6";
		case 7:		return "7";
		case 8:		return "8";
		case 9:		return "9";
		case 10:	return "*";
		case 11:	return "0";
		case 12:	return "#";

		case 13:	return "A";
		case 14:	return "B";
		case 15:	return "C";
		case 16:	return "D";

		case 17:	return "E";
		case 18:	return "F";
		case 19:	return "G";
		case 20:	return "H";

		case 21:	return "I";
		case 22:	return "J";
		case 23:	return "K";
		case 24:	return "L";

		default:	return " ";
	}
}

